@@ -31,20 +31,19 @@ LsBattery::LsBattery(QString name) :
     m_IsCharging = false;
     m_Percentage = 100;
     applySettings();
+
+#ifndef Q_WS_MAEMO_5
+    m_Percentage = 100;
+    setImage();
+#endif
 }
 
 void LsBattery::paint(QPainter *painter, const QStyleOptionGraphicsItem* option,
                       QWidget*)
 {
     painter->setClipRect(option->exposedRect);
-    if (m_IsCharging && !m_ChargingImage.isNull()){
-        painter->drawPixmap(0, 0, m_ChargingImage);
-    }else{
-        if (m_Images.count() > 0){
-            int idx = ceil(m_Percentage / (100.0 / (float)m_Images.count()) - 1.0);
-            painter->drawPixmap(0, 0, m_Images.at(idx));
-        }
-    }
+    if (!m_Image.isNull())
+        painter->drawPixmap(0, 0, m_Image);
 
     if (!m_IsCharging){
         QRectF rect = boundingRect();
@@ -86,14 +85,9 @@ void LsBattery::applySettings()
     QString images = getSetting("Images",
                                 "/opt/qtlockscreen/default/battery_0.png;/opt/qtlockscreen/default/battery_1.png;/opt/qtlockscreen/default/battery_2.png;/opt/qtlockscreen/default/battery_3.png;/opt/qtlockscreen/default/battery_4.png;/opt/qtlockscreen/default/battery_5.png;/opt/qtlockscreen/default/battery_6.png").toString();
 
-    m_Images.clear();
-    foreach (QString img, images.split(';')){
-        QPixmap i(img);
-        if (!i.isNull())
-            m_Images.append(i);
-    }
+    m_Images = images.split(';');
 
-    m_ChargingImage = QPixmap(getSetting("ChargingImage", "/opt/qtlockscreen/default/battery_charging.png").toString());
+    m_ChargingImage = getSetting("ChargingImage", "/opt/qtlockscreen/default/battery_charging.png").toString();
     m_PercColor = getColorSetting("PercentageColor", QColor(255,255,255));
     m_PercFont = getFontSetting("PercentageFont", QFont("", 15, QFont::Normal, false));
 }
@@ -172,10 +166,32 @@ void LsBattery::bmePropertyModifiedSlot(QDBusMessage)
     delete proc;
 
     qDebug() << "Battery percentage" << m_Percentage;
-    update();
+
+    setImage();
 }
 #endif
 
+void LsBattery::setImage()
+{
+    if (m_IsCharging){
+        if (m_CurrImage != m_ChargingImage){
+            m_CurrImage = m_ChargingImage;
+            m_Image = QPixmap(m_CurrImage);
+            update();
+        }
+    }else if (m_Images.count() > 0){
+        int idx = ceil(m_Percentage / (100.0 / (float)m_Images.count()) - 1.0);
+        if (idx < 0)
+            idx = 0;
+
+        if (m_CurrImage != m_Images.at(idx)){
+            m_CurrImage = m_Images.at(idx);
+            m_Image = QPixmap(m_CurrImage);
+            update();
+        }
+    }
+}
+
 float LsBattery::GetValue(QString file)
 {
     float value = 0.0;
