@@ -98,6 +98,7 @@ void LsWeather::startUpdate()
 {
 #ifdef Q_WS_MAEMO_5    
     if (!m_LastUpdate.isValid()){
+        qDebug() << "Weather widget first update";
         if (LockScreen::ScreenshotMode)
             updateWeather();
         else
@@ -134,65 +135,73 @@ void LsWeather::updateWeather()
 
     QString command = QString("/opt/qtlockscreen/weatherwidget.sh %1").arg(link);
     m_Process = new QProcess(this);
+    connect(m_Process, SIGNAL(finished(int,QProcess::ExitStatus)), this, SLOT(onProcessFinished(int,QProcess::ExitStatus)));
     m_Process->start(command, QIODevice::ReadOnly);
-    if (m_Process->waitForFinished()){
-        if (m_Process->exitCode() == 0){
-            m_Image = QPixmap(size().width(), size().height());
-            m_Image.fill(QColor(0,0,0,0));
-            QPainter painter(&m_Image);
+    if (LockScreen::ScreenshotMode)
+        m_Process->waitForFinished();
+}
 
-            if (m_ShowImage){
-                QImage tImage = QImage("/tmp/ywtemp.png");
-                if (!tImage.isNull())
-                    painter.drawImage(0,0, tImage);
-            }
+void LsWeather::onProcessFinished(int exitCode, QProcess::ExitStatus)
+{
+    if (exitCode == 0){
+        m_Image = QPixmap(size().width(), size().height());
+        m_Image.fill(QColor(0,0,0,0));
+        QPainter painter(&m_Image);
 
-            QString output = QString(m_Process->readAll());
-            qDebug() << "Weather widget command output:" << output;
-            m_LastUpdate = QDateTime::currentDateTime();
+        if (m_ShowImage){
+            QImage tImage = QImage("/tmp/ywtemp.png");
+            if (!tImage.isNull())
+                painter.drawImage(0,0, tImage);
+        }
 
-            QStringList lines = output.split("\n");
-            foreach (QString l, lines){
-                QStringList ele = l.split(" ", QString::SkipEmptyParts);
-                if (ele.count() >= 2){
-                    QString name = ele.at(0).toLower();
-                    int val = ele.at(1).toInt();
-                    QString temp;
-                    if (name == "current:" && !m_CurrentPos.isNull()){
-                        temp = QString("%1 %2").arg(m_CurrentText).arg(val).trimmed();
-                        painter.setFont(m_CurrentFont);
-                        painter.setPen(m_CurrentColor);
-                        painter.drawText(m_CurrentPos.x(), m_CurrentPos.y(), temp);
-                    }else if (name == "conditions:" && !m_ConditionsPos.isNull()){
-                        QString cond;
-                        for (int i=1; i<ele.count(); i++){
-                            cond.append(ele.at(i));
-                            cond.append(" ");
-                        }
-                        temp = QString("%1 %2").arg(m_ConditionsText).arg(cond).trimmed();
-                        painter.setFont(m_ConditionsFont);
-                        painter.setPen(m_ConditionsColor);
-                        painter.drawText(m_ConditionsPos.x(), m_ConditionsPos.y(), temp);
-                    }else if (name == "high:" && !m_HighPos.isNull()){
-                        temp = QString("%1 %2").arg(m_HighText).arg(val).trimmed();
-                        painter.setFont(m_HighFont);
-                        painter.setPen(m_HighColor);
-                        painter.drawText(m_HighPos.x(), m_HighPos.y(), temp);
-                    }else if (name == "low:" && !m_LowPos.isNull()){
-                        temp = QString("%1 %2").arg(m_LowText).arg(val).trimmed();
-                        painter.setFont(m_LowFont);
-                        painter.setPen(m_LowColor);
-                        painter.drawText(m_LowPos.x(), m_LowPos.y(), temp);
+        QString output = QString::fromUtf8(m_Process->readAll());
+        qDebug() << "Weather widget command output:" << output;
+        m_LastUpdate = QDateTime::currentDateTime();
+        qDebug() << "Update time:" << m_LastUpdate;
+
+        QStringList lines = output.split("\n");
+        foreach (QString l, lines){
+            QStringList ele = l.split(" ", QString::SkipEmptyParts);
+            if (ele.count() >= 2){
+                QString name = ele.at(0).toLower();
+                int val = ele.at(1).toInt();
+                QString temp;
+                if (name == "current:" && !m_CurrentPos.isNull()){
+                    temp = QString("%1 %2").arg(m_CurrentText).arg(val).trimmed();
+                    painter.setFont(m_CurrentFont);
+                    painter.setPen(m_CurrentColor);
+                    painter.drawText(m_CurrentPos.x(), m_CurrentPos.y(), temp);
+                }else if (name == "conditions:" && !m_ConditionsPos.isNull()){
+                    QString cond;
+                    for (int i=1; i<ele.count(); i++){
+                        cond.append(ele.at(i));
+                        cond.append(" ");
                     }
+                    temp = QString("%1 %2").arg(m_ConditionsText).arg(cond).trimmed();
+                    painter.setFont(m_ConditionsFont);
+                    painter.setPen(m_ConditionsColor);
+                    painter.drawText(m_ConditionsPos.x(), m_ConditionsPos.y(), temp);
+                }else if (name == "high:" && !m_HighPos.isNull()){
+                    temp = QString("%1 %2").arg(m_HighText).arg(val).trimmed();
+                    painter.setFont(m_HighFont);
+                    painter.setPen(m_HighColor);
+                    painter.drawText(m_HighPos.x(), m_HighPos.y(), temp);
+                }else if (name == "low:" && !m_LowPos.isNull()){
+                    temp = QString("%1 %2").arg(m_LowText).arg(val).trimmed();
+                    painter.setFont(m_LowFont);
+                    painter.setPen(m_LowColor);
+                    painter.drawText(m_LowPos.x(), m_LowPos.y(), temp);
                 }
             }
+        }
 
-            update();
-        }else
-            qDebug() << "  Process exited with code" << m_Process->exitCode();
+        update();
     }else
-        qDebug() << "  Failed to wait for process to finish";
+        qDebug() << "  Process exited with code" << m_Process->exitCode();
 
-    delete m_Process;
+    disconnect(m_Process, SIGNAL(finished(int,QProcess::ExitStatus)), this, SLOT(onProcessFinished(int,QProcess::ExitStatus)));
+    m_Process->close();
+    //delete m_Process;
+    m_Process->deleteLater();
     m_Process = NULL;
 }
